import json


class ArchiveException(Exception):
    pass

class ArchiveAlreadyExistsException(ArchiveException):
    pass

class NotAnArchiveException(ArchiveException):
    pass

class Archive(object):
    def __init__(self, root):
        self.root = root
        self.settings_path = root / "selvage.json"
        self.settings = {}
        self.loaded = False
        self.load()
        # if not os.path.exists(self.settings_path):
        #     if create:
        #         self.settings = {}
        #         self.save()
        #     else:
        #         raise NotAnArchiveException("can't open selvage.json")
        # else:
        #     if exist_ok:
        #         with open(self.settings.path) as fin:
        #             self.settings = json.load(fin)
        #     else:
        #         raise ArchiveAlreadyExistsException()

    def add_pattern (self, input, output):
        if "patterns" not in self.settings:
            self.settings['patterns'] = []
        self.settings['patterns'].append((input, output))
        self.save()

    def set_setting (self, key, value):
        self.settings[key] = value
        self.save()

    def get_setting (self, key, default=None):
        return self.settings.get(key, default)

    def load (self):
        if not self.loaded:
            if self.settings_path.exists():
                with open(self.settings_path) as fin:
                    self.settings = json.load(fin)
            else:
                self.settings = {}
            self.loaded = True

    def save(self):
        with open(self.settings_path, "w") as fout:
            print(json.dumps(self.settings, indent=2), file=fout)

    def patterns (self):
        if 'patterns' in self.settings:
            for key, value in self.settings['patterns']:
                yield key, value