import os, sys
from urllib.parse import urlparse
import requests


EXT = {
    "text/html": "html",
    "text/css": "css",
    "image/jpeg": "jpg",
    "image/png": "png",
    "image/gif": "gif",
    "image/svg+xml": "svg",
    "application/javascript": "js",
    "text/javascript": "js",
    "video/webm": "webm",
    "font/ttf": "ttf",
    "font/woff": "woff",
    "font/woff2": "woff2",
    "application/vnd.ms-fontobject": "eot",
    "application/font-woff": "woff",
    "application/rss+xml": "rss",
}

def memoize(f):
    memo = {}

    def helper(x):
        if x not in memo:
            memo[x] = f(x)
        return memo[x]

    return helper

@memoize
def ext_for(url):
    # try / allow simple extension test to override
    parts = urlparse(url)
    if parts.path:
        ext = os.path.splitext(parts.path)[1].lstrip(".").lower()
        if ext: # in EXT.values():
            return ext
    try:
        r = requests.head(url, verify=False)
        ct = r.headers["content-type"].split(";")[0]
        if ct not in EXT:
            print(
                f"Warning, unknown extension for content-type {ct}, using bin",
                file=sys.stderr,
            )
        return EXT.get(ct, "bin")
    except Exception as e:
        print(f"Exception {url}: {e}", file=sys.stderr)
        return "bin"
