from wayback import WaybackClient
import html5lib
from urllib.parse import urljoin, urlparse, quote as urlquote
import requests
import os
from pathlib import Path
from xml.etree import ElementTree as ET


"""
future:
use centralized functions to
* map URL to local path
* produce relative links
* also handle "internal" links?

"""


def wayback_repair(path, output_path=".", baseurl=None, force=False):
    client = WaybackClient()
    with open(path) as fin:
        t = html5lib.parse(fin, namespaceHTMLElements=False)
    rewrite_count = 0
    for link in t.findall(".//img[@src]"):
        src = link.attrib.get("src")
        if baseurl:
            src = urljoin(baseurl, src)
        r = requests.get(src)
        if r.status_code != 200:
            print (src, r.status_code)

            src_parsed = urlparse(src)
            basename = os.path.basename(src_parsed.path)
            local_path = Path(output_path) / basename
            rel_path = urlquote(f"{output_path}/{basename}")
            link.attrib['src'] = urlquote(rel_path)
            rewrite_count += 1
            if local_path.exists() and not force:
                print (f"{local_path} exists, skipping")
                continue
            results = client.search(src)
            print (f"Searching wayback...")
            for result in results:
                print (f"Got a result from {result.timestamp}...")
                print (f"Saving {result.raw_url} to {local_path}")
                try:
                    with requests.get(result.raw_url, stream=True) as r:
                        r.raise_for_status()
                        with open(local_path, 'wb') as f:
                            for chunk in r.iter_content(chunk_size=8192):
                                f.write(chunk)
                    break
                except Exception as e:
                    print (f"Exception: {e}")
                    
    if rewrite_count:
        print (f"Rewrote {rewrite_count} links")
        print (f"Saving {path}")
        with open(path, "w") as fout:
            print (ET.tostring(t, method="html", encoding="unicode"), file=fout)